#include <apr_hash.h>
#include <apr_strings.h>
#include <httpd.h>
#include <http_config.h>

static apr_hash_t *mime_extensions = NULL;

/**
 * Initializes the internal structures with the mapping
 * @param pool     The pool
 * @param map_path The path to the mime file
 */
void dav_dpm_mime_init(apr_pool_t *pool, const char *map_path)
{
  ap_configfile_t *mime_cfg;
  char             line[MAX_STRING_LEN];

  /* Create the hash table */
  mime_extensions = apr_hash_make(pool);

  /* Open the mime types file */
  if (ap_pcfg_openfile(&mime_cfg, pool, map_path) != APR_SUCCESS) {
    return;
  }

  /* Read line by line */
   while (!(ap_cfg_getline(line, MAX_STRING_LEN, mime_cfg))) {
     const char *ll = line, *ct;

     if (line[0] == '#')
       continue;

     ct = ap_getword_conf(pool, &ll);

     while(ll[0]) {
       char *ext = ap_getword_conf(pool, &ll);
       ap_str_tolower(ext);
       apr_hash_set(mime_extensions, ext, APR_HASH_KEY_STRING, ct);
     }
   }

  ap_cfg_closefile(mime_cfg);
}

/**
 * Returns the mime type associated with the filename's extension
 * If no extension is found, or not correspondance, NULL will be returned
 * @param buffer   Where to put the value
 * @param maxlen   The size of the buffer
 * @param filename The file name
 * @return buffer if found, NULL if not
 */
const char *dav_dpm_mime_get(char *buffer, size_t maxlen, const char *filename)
{
  char *p;

  if (!mime_extensions)
    return NULL;

  if (!(p = strrchr(filename, '.')))
    return NULL;

  strncpy(buffer, p + 1, maxlen);
  ap_str_tolower(buffer);

  p = apr_hash_get(mime_extensions, buffer, APR_HASH_KEY_STRING);
  if (p) {
    strncpy(buffer, p, maxlen);
    return buffer;
  }
  else {
    return NULL;
  }
}
