#include <apr_strings.h>
#include <ctype.h>
#include <json/json.h>
#include "replicas.h"

char* dav_ns_serialize_replicas(apr_pool_t *pool,
                                int nreplicas,
                                struct Cns_filereplica *replicas,
                                short brief)
{
  int   i;
  char *serial;

  serial = "[\n";

  for (i = 0; i < nreplicas; ++i) {
    serial = apr_psprintf(pool,
"%s"
"{\n"
"\t\"server\"    : \"%s\",\n"
"\t\"url\"       : \"%s\",\n"
"\t\"atime\"     : %lu%s\n", serial, replicas[i].host, replicas[i].sfn,
                             replicas[i].atime, brief?"":",");

    if (!brief) {
      serial = apr_psprintf(pool,
"%s"
"\t\"filesystem\": \"%s\",\n"
"\t\"pool\"      : \"%s\",\n"
"\t\"status\"    : \"%c\",\n"
"\t\"type\"      : \"%c\",\n", serial, replicas[i].fs, replicas[i].poolname,
                               replicas[i].status?replicas[i].status:'0',
                               replicas[i].f_type?replicas[i].f_type:'0');
    }

    if (i + 1 < nreplicas)
      serial = apr_pstrcat(pool, serial, "},\n", NULL);
    else
      serial = apr_pstrcat(pool, serial, "}\n", NULL);
  }

  serial = apr_pstrcat(pool, serial, "]", NULL);

  return serial;
}



dav_error* dav_ns_deserialize_replicas(apr_pool_t *pool,
                                       const char* json_str,
                                       struct dav_ns_replica_array* replicas)
{
  struct json_object *json = json_tokener_parse(json_str);
  int                 i;

  /* Just in case (see later) */
  if (json == NULL)
    return dav_new_error(pool, HTTP_CONFLICT, 0,
                         "Could not parse the JSON string");

  /* Weirdness in the error handling of json_tokener_parser :/  */
  enum json_tokener_error error = (enum json_tokener_error)(-(ptrdiff_t)json);
  if (error > json_tokener_success && error <= json_tokener_error_parse_comment)
    return dav_new_error(pool, HTTP_CONFLICT, 0,
                         "Could not parse the JSON string");

  /* Must be an array! */
  if (json_object_get_type(json) != json_type_array)
    return dav_new_error(pool, HTTP_CONFLICT, 0,
                         "First-level JSON Object must be an array");

  /* Initialize the array */
  replicas->nreplicas = json_object_array_length(json);
  replicas->replicas  = apr_pcalloc(pool, sizeof(struct Cns_filereplica) * replicas->nreplicas);
  replicas->action    = apr_pcalloc(pool, sizeof(char) * replicas->nreplicas);

  /* Iterate and copy */
  for (i = 0; i < replicas->nreplicas; ++i) {
     struct json_object *json_replica = json_object_array_get_idx(json, i);

     /* Get the json objects */
     struct json_object *server = json_object_object_get(json_replica, "server");
     struct json_object *url    = json_object_object_get(json_replica, "url");
     struct json_object *fs     = json_object_object_get(json_replica, "filesystem");
     struct json_object *pool   = json_object_object_get(json_replica, "pool");
     struct json_object *status = json_object_object_get(json_replica, "status");
     struct json_object *type   = json_object_object_get(json_replica, "type");
     struct json_object *action = json_object_object_get(json_replica, "action");
     struct json_object *atime  = json_object_object_get(json_replica, "atime");
     struct json_object *ltime  = json_object_object_get(json_replica, "ltime");

     /* Get the actual values */
     if (server != NULL && json_object_get_type(server) == json_type_string)
       strncpy(replicas->replicas[i].host, json_object_get_string(server), CA_MAXHOSTNAMELEN);
     if (url != NULL && json_object_get_type(url) == json_type_string)
       strncpy(replicas->replicas[i].sfn, json_object_get_string(url), CA_MAXSFNLEN);
     if (fs != NULL && json_object_get_type(fs) == json_type_string)
       strncpy(replicas->replicas[i].fs, json_object_get_string(fs), 80);
     if (pool != NULL && json_object_get_type(pool) == json_type_string)
       strncpy(replicas->replicas[i].poolname, json_object_get_string(pool), CA_MAXPOOLNAMELEN);
     if (status != NULL && json_object_get_type(status) == json_type_string)
       replicas->replicas[i].status = json_object_get_string(status)[0];
     if (type != NULL && json_object_get_type(type) == json_type_string)
       replicas->replicas[i].f_type = json_object_get_string(type)[0];
     if (atime != NULL && json_object_get_type(atime) == json_type_boolean)
       replicas->replicas[i].atime = json_object_get_boolean(atime);
      if (ltime != NULL && json_object_get_type(ltime) == json_type_int)
       replicas->replicas[i].ltime = json_object_get_int(ltime);

     if (action != NULL && json_object_get_type(action) == json_type_string)
       replicas->action[i] = toupper(json_object_get_string(action)[0]);
     else
       replicas->action[i] = 'M';
  }

  /* Weird name for a free function... */
  json_object_put(json);

  return NULL;
}
